@@ -67,6 +67,7 @@ import com.openexchange.mobile.api.facade.auth.SessionData;
 import com.openexchange.mobile.api.facade.endpoints.requests.MailsDeleteRequestBody;
 import com.openexchange.mobile.api.facade.endpoints.requests.MailsRequestBody;
 import com.openexchange.mobile.api.facade.endpoints.requests.MoveMailsRequestBody;
+import com.openexchange.mobile.api.facade.endpoints.requests.NewMailsRequestBody;
 import com.openexchange.mobile.api.facade.endpoints.requests.SetColorsRequestBody;
 import com.openexchange.mobile.api.facade.endpoints.requests.SetFlagsRequestBody;
 import com.openexchange.mobile.api.facade.endpoints.responses.DeletedMailsResponseBody;
@@ -105,6 +106,8 @@ public class MailsEndpoint {
 
     private static final String FIELD_MAIL_IDS = "mailIds";
 
+    private static final String FIELD_LAST_RECEIVED_DATE = "lastReceivedDate";
+
     private static final MapFunction<MailThread, ThreadCto> threadMapFunction = new MapFunction<MailThread, ThreadCto>() {
         @Override
         public ThreadCto map(MailThread thread) {
@@ -130,14 +133,30 @@ public class MailsEndpoint {
         SessionData sessionData = RequestUtil.getSessionData(request);
         MailsRequestBody requestBody = JacksonUtil.parse(rawRequestBody, MailsRequestBody.class);
         RequestUtil.checkParam(FIELD_FOLDER_ID, requestBody.getFolderId());
-        Boolean threadedValue = requestBody.getThreaded();
-        boolean threaded = threadedValue == null || threadedValue.booleanValue();
+        boolean threaded = RequestUtil.getBooleanParameter(requestBody.getThreaded(), true);
         List<MailThread> threads = mailsService.getStubs(sessionData, requestBody.getFolderId(), threaded);
         List<ThreadCto> threadCtos = ListUtil.map(threads, threadMapFunction);
         MailThreadsResponseBody threadsResponseBody = new MailThreadsResponseBody(threadCtos);
         return RequestUtil.createCachableResponse(request, threadsResponseBody);
     }
 
+    @POST
+    @Path("/stubs/new")
+    @Consumes(MediaType.APPLICATION_JSON)
+    @Produces(MediaType.APPLICATION_JSON)
+    public Response getNewStubs(@Context HttpServletRequest request, String rawRequestBody) {
+        log.info("Retrieving new stubs");
+        SessionData sessionData = RequestUtil.getSessionData(request);
+        NewMailsRequestBody requestBody = JacksonUtil.parse(rawRequestBody, NewMailsRequestBody.class);
+        RequestUtil.checkParam(FIELD_FOLDER_ID, requestBody.getFolderId());
+        RequestUtil.checkParam(FIELD_LAST_RECEIVED_DATE, requestBody.getLastReceivedDate());
+        boolean threaded = RequestUtil.getBooleanParameter(requestBody.getThreaded(), true);
+        List<MailThread> threads = mailsService.getNewStubs(sessionData, requestBody.getFolderId(), requestBody.getLastReceivedDate(), threaded);
+        List<ThreadCto> threadCtos = ListUtil.map(threads, threadMapFunction);
+        MailThreadsResponseBody responseBody = new MailThreadsResponseBody(threadCtos);
+        return RequestUtil.jsonResponse(responseBody);
+    }
+
     @POST
     @Path("/envelopes")
     @Consumes(MediaType.APPLICATION_JSON)
@@ -154,6 +173,22 @@ public class MailsEndpoint {
         return RequestUtil.jsonResponse(mailResponseBody);
     }
 
+    @POST
+    @Path("/envelopes/new")
+    @Consumes(MediaType.APPLICATION_JSON)
+    @Produces(MediaType.APPLICATION_JSON)
+    public Response getNewEnvelopes(@Context HttpServletRequest request, String rawRequestBody) {
+        log.info("Retrieving new stubs");
+        SessionData sessionData = RequestUtil.getSessionData(request);
+        NewMailsRequestBody requestBody = JacksonUtil.parse(rawRequestBody, NewMailsRequestBody.class);
+        RequestUtil.checkParam(FIELD_FOLDER_ID, requestBody.getFolderId());
+        RequestUtil.checkParam(FIELD_LAST_RECEIVED_DATE, requestBody.getLastReceivedDate());
+        List<Mail> mails = mailsService.getNewEnvelopes(sessionData, requestBody.getFolderId(), requestBody.getLastReceivedDate());
+        List<MailCto> mailCtos = ListUtil.map(mails, mailMapFunction);
+        MailsResponseBody responseBody = new MailsResponseBody(mailCtos);
+        return RequestUtil.jsonResponse(responseBody);
+    }
+
     @POST
     @Path("/complete")
     @Consumes(MediaType.APPLICATION_JSON)
